/*! For license information please see lander.bundle.js.LICENSE.txt */
(()=>{var e,t,o,n,i,r={64:function(e,t){var o,n,i;n=[],void 0===(i="function"==typeof(o=function(){"use strict";if("undefined"==typeof window||void 0===window.navigator||void 0===window.navigator.userAgent||void 0===window.navigator.mimeTypes)return!1;let e="2.2.3",t=window.navigator,o=window.navigator.userAgent,n="ActiveXObject"in window,i=void 0!==window.Promise,r=void 0!==t.mimeTypes["application/pdf"],a=void 0!==t.platform&&"MacIntel"===t.platform&&void 0!==t.maxTouchPoints&&t.maxTouchPoints>1||/Mobi|Tablet|Android|iPad|iPhone/.test(o),d=!a&&void 0!==t.vendor&&/Apple/.test(t.vendor)&&/Safari/.test(o),s=!(a||!/irefox/.test(o))&&parseInt(o.split("rv:")[1].split(".")[0],10)>18,p=function(e){var t;try{t=new ActiveXObject(e)}catch(e){t=null}return t},l=function(){return!(!p("AcroPDF.PDF")&&!p("PDF.PdfCtrl"))},c=!a&&(s||r||n&&l()),f=function(e){let t,o="";if(e){for(t in e)e.hasOwnProperty(t)&&(o+=encodeURIComponent(t)+"="+encodeURIComponent(e[t])+"&");o&&(o="#"+o,o=o.slice(0,o.length-1))}return o},u=function(e,t){return t||console.log("[PDFObject] "+e),!1},m=function(e){for(;e.firstChild;)e.removeChild(e.firstChild)},y=function(e){let t=document.body;return"string"==typeof e?t=document.querySelector(e):void 0!==window.jQuery&&e instanceof jQuery&&e.length?t=e.get(0):void 0!==e.nodeType&&1===e.nodeType&&(t=e),t},b=function(e,t,o,n,i,r){m(e);let a=n+"?file="+encodeURIComponent(t)+o,d=document.createElement("div"),s=document.createElement("iframe");return s.src=a,s.className="pdfobject",s.type="application/pdf",s.frameborder="0",i&&(s.id=i),r||(d.style.cssText="position: absolute; top: 0; right: 0; bottom: 0; left: 0;",s.style.cssText="border: none; width: 100%; height: 100%;",e.style.position="relative",e.style.overflow="auto"),d.appendChild(s),e.appendChild(d),e.classList.add("pdfobject-container"),e.getElementsByTagName("iframe")[0]},g=function(e,t,o,n,i,r,a,d,s){m(t);let p=document.createElement(e);if(p.src=n+i,p.className="pdfobject",p.type="application/pdf",d&&(p.id=d),!s){let t="embed"===e?"overflow: auto;":"border: none;";o&&o!==document.body?t+="width: "+r+"; height: "+a+";":t+="position: absolute; top: 0; right: 0; bottom: 0; left: 0; width: 100%; height: 100%;",p.style.cssText=t}return t.classList.add("pdfobject-container"),t.appendChild(p),t.getElementsByTagName(e)[0]},h=function(e,t,o){let n=t||!1,r=o||{},s="string"==typeof r.id?r.id:"",p=r.page||!1,l=r.pdfOpenParams||{},m=r.fallbackLink||!0,h=r.width||"100%",v=r.height||"100%",w="boolean"!=typeof r.assumptionMode||r.assumptionMode,P="boolean"==typeof r.forcePDFJS&&r.forcePDFJS,T="boolean"==typeof r.supportRedirect&&r.supportRedirect,D="boolean"==typeof r.omitInlineStyles&&r.omitInlineStyles,x="boolean"==typeof r.suppressConsole&&r.suppressConsole,C="boolean"==typeof r.forceIframe&&r.forceIframe,F=r.PDFJS_URL||!1,j=y(n),I="",R="",S="<p>This browser does not support inline PDFs. Please download the PDF to view it: <a href='[url]'>Download PDF</a></p>";return"string"!=typeof e?u("URL is not valid",x):j?(p&&(l.page=p),R=f(l),P&&F?b(j,e,R,F,s,D):c||w&&i&&!a?g(C||T&&d?"iframe":"embed",j,t,e,R,h,v,s,D):F?b(j,e,R,F,s,D):(m&&(I="string"==typeof m?m:S,j.innerHTML=I.replace(/\[url\]/g,e)),u("This browser does not support embedded PDFs",x))):u("Target element cannot be determined",x)};return{embed:function(e,t,o){return h(e,t,o)},pdfobjectversion:e,supportsPDFs:c}})?o.apply(t,n):o)||(e.exports=i)}},a={};function d(e){if(a[e])return a[e].exports;var t=a[e]={exports:{}};return r[e].call(t.exports,t,t.exports,d),t.exports}e=d(64),t="lander-pdf-container",o=document.getElementById(t).dataset.pdfPath,n=new URLSearchParams(document.location.search.substring(1)).get("page"),i={},n&&(i.page=n),e.embed(o,"#".concat(t),i)})();